// BlockIoPage.cpp : implementation file
//

#include "stdafx.h"
#include "DiiDemo.h"
#include "BlockIoPage.h"

#include "DiiDemoDlg.h"
#include "Dii.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

		//
		// This message we'll use for receiving notifications from the
		// DII.
#define MSG_RECEIVED_NOTIFICATION (WM_USER+1)

/////////////////////////////////////////////////////////////////////////////
// CBlockIoPage property page

IMPLEMENT_DYNCREATE(CBlockIoPage, CPropertyPage)

CBlockIoPage::CBlockIoPage() : CPropertyPage(CBlockIoPage::IDD)
{
	//{{AFX_DATA_INIT(CBlockIoPage)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

		//
		// Fill the buffer with ascending data:

	for (int iCounter = 0;iCounter < BUFFER_SIZE;iCounter++)
		m_byOutputBuffer[iCounter] = iCounter;
}

CBlockIoPage::~CBlockIoPage()
{
}

void CBlockIoPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBlockIoPage)
	DDX_Control(pDX, IDC_EVENT_LOG, m_lstbxEventLog);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CBlockIoPage, CPropertyPage)
	//{{AFX_MSG_MAP(CBlockIoPage)
	ON_BN_CLICKED(IDC_ENABLE_TRANSFER, OnEnableTransfer)
	ON_BN_CLICKED(IDC_SEND_DATA, OnSendData)
	//}}AFX_MSG_MAP
	ON_MESSAGE (MSG_RECEIVED_NOTIFICATION,OnReceivedNotification)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBlockIoPage message handlers

void CBlockIoPage::OnEnableTransfer() 
{
	BOOL bIsEnabled = IsDlgButtonChecked (IDC_ENABLE_TRANSFER);

		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// We enable the input and output functions for the DII Block I/O 
		// routines and enable notification handling via Windows Messages.
		// We could also receive messages via a callback, but for demonstration
		// purposes, we use window messages here, they are also easier to 
		// handle, since they are already synchronized with the rest of the code.
		//
		// ------------------------------------------------------------------

	BOOL bSucceeded = FALSE;
	
	do 
	{

		if (bIsEnabled)
		{

				//
				// Enable the automatic background block I/O:
				//

			//
			// Setup notification method:
			if (!DiiSetNotificationMethodHwnd (
							((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
							m_hWnd,
							MSG_RECEIVED_NOTIFICATION,
							0
						))	break;

			//
			// Request notifications for data blocks being sent or received
			if (!DiiRequestNotification (
						((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
						DII_DATA_BLOCK_READ | DII_DATA_BLOCK_SENT, 
						0 )) break;

			//
			// Now initiate block I/O for the given device:
			// first input:
			if (!DiiPrepareBlockInput (
						((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
						0,					// for the first channel only
						DII_TRIGGER_SOFTWARE,	// std. software trigger
						0,					// use the default value of the device
						1024,				// request 1kB buffers (may be changed by DII)
						NULL				// no direct storage to a file!
					))	break;

			//
			// start the block input:
			if (!DiiStartBlockInput	(
						((CDiiDemoDlg *)GetParent())->GetDeviceHandle()
					)) break;


			//
			// Now configure the block output:
			if (!DiiPrepareBlockOutput (
						((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
						0,					// for the first channel only
						DII_TRIGGER_SOFTWARE,	// std. software trigger
						0					// use the default value of the device
				)) break;


			//
			// start the block output:
			if (!DiiStartBlockInput	(
						((CDiiDemoDlg *)GetParent())->GetDeviceHandle()
					)) break;
		} else
		{
				//
				// Disable the automatic background block I/O:
				//

			DiiEndBlockInput	(
					((CDiiDemoDlg *)GetParent())->GetDeviceHandle()
				);

			DiiEndBlockOutput	(
					((CDiiDemoDlg *)GetParent())->GetDeviceHandle()
				);

				//
				// Also disable notifications:
				//

			DiiCancelNotification (
					((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
					0
				);

				//
				// Clear the notification listbox:

			m_lstbxEventLog.ResetContent();

		}

		bSucceeded = TRUE;

	} while (0);

	if (!bSucceeded)
	{
//		CheckDlgButton (IDC_ENABLE_TRANSFER,!bSucceeded);
		AfxMessageBox ("Could not initiate Block I/O on this device.\nProbably not supported.",MB_OK | MB_ICONSTOP);
	}
		
		//
		// Enable the controls accordingly
	GetDlgItem (IDC_SEND_DATA)->EnableWindow (bSucceeded && bIsEnabled);
	GetDlgItem (IDC_EVENT_LOG)->EnableWindow (bSucceeded && bIsEnabled);
}

void CBlockIoPage::OnSendData() 
{
		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// We need to send some data down to the device. The transmission
		// is in the background, so it's important that we don't use
		// a buffer on the stack, but one that exists outside this function 
		// call. We use the static member "m_byOutputBuffer" for that.
		//
		// ------------------------------------------------------------------

	if (!DiiOutputBlock (
			((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
			0,						// output to the first channel
			m_byOutputBuffer,		// the buffer to output
			BUFFER_SIZE,			// the size of the buffer
			FALSE					// no, don't send the buffer contineously
		))
	{
		AfxMessageBox ("Could not send data!");
	}
}

		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// This message call back is called whenever a notification was
		// received. We add a line to the listbox, showing when and what
		// was received.
		// 
		// ------------------------------------------------------------------

LONG CBlockIoPage::OnReceivedNotification (WPARAM,LPARAM lParam)
{
		//
		// The LPARAM parameter points to our LPDII_NOFICATION_DATA structure

	LPDII_NOTIFICATION_DATA lpNotifData = LPDII_NOTIFICATION_DATA (lParam);

		//
		// Display what this notification is all about:

	CString strMsg;

	CTime curTime = CTime::GetCurrentTime();

	CString strTimeStamp = curTime.Format ("%X");
	

	if ((lpNotifData->dwNotificationMask & DII_DATA_BLOCK_READ) != 0)
	{
		strMsg.Format ("%s : Received %d bytes from device.",
					LPCTSTR (strTimeStamp),
					lpNotifData->dwBufferSize
				);
	} else
	{
		strMsg.Format ("%s : Sent %d bytes to the device.",
					LPCTSTR (strTimeStamp),
					lpNotifData->dwBufferSize
				);
	}

		// add the string (in front!)
	int iIndex = m_lstbxEventLog.InsertString (0,strMsg);
	m_lstbxEventLog.SetTopIndex (iIndex);

		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// We MUST release the notification data after processing it, so the
		// DII can automatically reuse buffers!
		// 
		// ------------------------------------------------------------------

	DiiReleaseNotificationData (
					((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
					lpNotifData
				);
	return 1;
}
