// PulseCountPage.cpp : implementation file
//

#include "stdafx.h"
#include "DiiDemo.h"
#include "PulseCountPage.h"
#include "DiiDemoDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPulseCountPage property page

IMPLEMENT_DYNCREATE(CPulseCountPage, CPropertyPage)

CPulseCountPage::CPulseCountPage() : CPropertyPage(CPulseCountPage::IDD)
{
	//{{AFX_DATA_INIT(CPulseCountPage)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CPulseCountPage::~CPulseCountPage()
{
}

void CPulseCountPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPulseCountPage)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPulseCountPage, CPropertyPage)
	//{{AFX_MSG_MAP(CPulseCountPage)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPulseCountPage message handlers

BOOL CPulseCountPage::OnKillActive() 
{
	//
	// If this page is no longer active, kill our timer again:
	//

	KillTimer (1);
	
		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// Disable pulse counting again, and also cancel any notifications
		//
		// ------------------------------------------------------------------

	DiiDisablePulseCounting (((CDiiDemoDlg *)GetParent())->GetDeviceHandle());

	DiiCancelNotification (((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),0);
	

	return CPropertyPage::OnKillActive();
}

BOOL CPulseCountPage::OnSetActive() 
{
	//
	// As soon as we're active, attempt to initialize the pulse counting
	// interface, and start the timer if successful:
	//

		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// Enable Pulse Counting on the specified device. We count on all ports.
		//
		// ------------------------------------------------------------------

	if (!DiiEnablePulseCounting (
		((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),	
		0,	// start port (default: the first one)
		0,	// number of ports (default: all)
		0))	// Reserved
	{
			//
			// Pulse counting does not seem to be supported for the device:
			//
		for (int counter = 0;counter < 16;counter+=2)
			GetDlgItem (IDC_CHANNEL_1_COUNT + counter)->EnableWindow (FALSE);

		for (counter = 0;counter < 4;counter++)
			GetDlgItem (IDC_CHANNEL_1_SECCOUNT + counter)->EnableWindow (FALSE);

		return FALSE;
	}


		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// Install a callback, if one counter reaches zero, and 
		// request notifications if a counter has reached zero.
		// For this, we don't care about the return argument.
		//
		// ------------------------------------------------------------------

	DiiSetNotificationMethodCallBack (((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
										CounterReachedZero);

	DiiRequestNotification (((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
							DII_COUNTER_ZERO,
							0);


		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// Configure the last 4 counters as downcounters, with different
		// initial values:
		// Channel 4,6 don't reset themselves but stay at 0,
		// Channel 5,7 reset to the original at a terminal count.
		//
		// ------------------------------------------------------------------

	DiiSetDownPulseCounter (
				((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
				4,
				1000,
				FALSE
			);
	DiiSetDownPulseCounter (
				((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
				5,
				100,
				TRUE
			);
	DiiSetDownPulseCounter (
				((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
				6,
				10,
				FALSE
			);
	DiiSetDownPulseCounter (
				((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
				7,
				1,
				TRUE
			);

		//
		// Clear the accumulative counters:
	for (int counter = 0;counter < 4;counter++)
		m_dwChannelPulseCount[counter] = 0;

	SetTimer (1,1000,NULL);
	
	return CPropertyPage::OnSetActive();
}

void CPulseCountPage::OnTimer(UINT nIDEvent) 
{
	//
	// In this function, update the pulse counts for all channels:
	//

		//
		// Update the UP-Counters:
	
	for (int counter = 0;counter < 4;counter++)
	{
		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// Query the current value of a counter. The counter is reset
		// while reading.
		//
		// ------------------------------------------------------------------
		
		DWORD dwCounterValue;
		DWORD dwMinTime;
		DWORD dwMaxTime;

		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// Query the current value of a counter. The counter is not reset
		// while reading.
		//
		// ------------------------------------------------------------------

		if (DiiGetPulseCounterValue (
					((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
					counter,
					&dwCounterValue,
					&dwMinTime,
					&dwMaxTime,
					TRUE))
		{
			SetDlgItemInt (IDC_CHANNEL_1_SECCOUNT + 2*counter,dwCounterValue,FALSE);

			m_dwChannelPulseCount[counter] += dwCounterValue;

			SetDlgItemInt (IDC_CHANNEL_1_COUNT + 2*counter,m_dwChannelPulseCount[counter],FALSE);
		}

	}

		//
		// Update the Down-Counters:

	for (counter = 4;counter < 8;counter++)
	{
		DWORD dwCounterValue;
		DWORD dwMinTime;
		DWORD dwMaxTime;

		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// Query the current value of a counter. The counter is not reset
		// while reading.
		//
		// ------------------------------------------------------------------

		if (DiiGetPulseCounterValue (
					((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
					counter,
					&dwCounterValue,
					&dwMinTime,
					&dwMaxTime,
					FALSE))
		{
			SetDlgItemInt (IDC_CHANNEL_1_COUNT + 2*counter,dwCounterValue,FALSE);
		}

	}


	CPropertyPage::OnTimer(nIDEvent);
}


void CPulseCountPage::CounterReachedZero(LPDII_NOTIFICATION_DATA lpNotifData)
{

		// In here, depending on which counter reached zero, sound
		// different beeps:

	if (lpNotifData->dwNotificationParam & 0x0000010)
		MessageBeep (MB_OK);
	if (lpNotifData->dwNotificationParam & 0x0000020)
		MessageBeep (MB_ICONQUESTION);
	if (lpNotifData->dwNotificationParam & 0x0000040)
		MessageBeep (MB_ICONHAND);
	if (lpNotifData->dwNotificationParam & 0x0000080)
		MessageBeep (MB_ICONEXCLAMATION);


}