// 
// File: DII.H
//
// Description: Declarations for Dynamic Industrial Interface
//              Library
//				For Windows 95 and Windows NT
//
//              Standard Version
//
// Copyright (C) 97-99 NT-ware Systemprogrammierungs GmbH
// All rights reserved.
//
//

#ifndef DII_H
#define DII_H

#include <windows.h>

#if defined (__cplusplus)
extern "C" {
#endif

#ifndef LPBOOL
#define LPBOOL BOOL FAR *
#endif

#define DIIEXPORT _stdcall 

// -------------------------------------------------------------------------
//					Functions to open and close Devices
// -------------------------------------------------------------------------

// DiiOpenDevice opens an Device for later use


HANDLE DIIEXPORT DiiOpenDevice 
					(	DWORD,		// Device ID
						DWORD,		// index of Device
						BOOL );		// open exclusively

// DiiOpenNamedDevice opens a device by it's name

HANDLE DIIEXPORT DiiOpenNamedDevice (
						LPCTSTR,	// Device Name
						BOOL );		// open exclusively

// DiiCloseDevice closes an Device.

BOOL DIIEXPORT DiiCloseDevice 
					( HANDLE );		// handle of previously 
									// opened Device

	// Return Value: TRUE, of Device was closed successfully, FALSE otherwise,

// -------------------------------------------------------------------------
//					Functions to enumerate or browse Devices
// -------------------------------------------------------------------------

// DiiSelectDevice opens a window to display a list or
// tree of Devices that are currently installed. The 
// user may select one.

BOOL DIIEXPORT DiiSelectDevice (
						HWND,		// handle of parent window or NULL
						LPTSTR,		// points to the name of the adapter selected
						DWORD);		// points to the maximum string length of the adapter


// DiiGetInstalledDevice may be used by applications
// to enumerate all Devices that are installed in the
// system. For each Device, the function returns the
// type and index within the system.
// The function returns TRUE if an Device was
// returnes successfully, FALSE otherwise.

BOOL DIIEXPORT DiiGetInstalledDevice (
						DWORD,		// index of Device to return
						LPTSTR,		// points to the name of the adapter selected
						DWORD);		// points to the maximum string length of the adapter

// -------------------------------------------------------------------------
//			Functions to retrieve information about an Device
// -------------------------------------------------------------------------

// DiiGetNumberOfDigitalChannels returns the number
// of digital channels the specified Device offers

DWORD DIIEXPORT DiiGetNumberOfDigitalChannels (
						HANDLE ); // a handle of an Device previously opened

	//
	// Returns the number of in and output channels:
BOOL DIIEXPORT DiiGetNumberOfDigitalInOutChannels (
						HANDLE ,	// a handle of an Device previously opened
						LPDWORD,	// a pointer to the number of INPUT channels
						LPDWORD);	// a pointer to the number of OUTPUT channels

	// returns the total number of digital
	// channels the card offers. (each channel is one bit)

DWORD DIIEXPORT DiiGetNumberOfAnalogChannels (
						HANDLE ); // a handle of an Device previously opened

	//
	// Returns the number of in and output channels:
BOOL DIIEXPORT DiiGetNumberOfAnalogInOutChannels (
						HANDLE ,	// a handle of an Device previously opened
						LPDWORD,	// a pointer to the number of INPUT channels
						LPDWORD);	// a pointer to the number of OUTPUT channels
						
	// returns the total number of analog
	// channels the card offers

DWORD DIIEXPORT DiiGetResolution(
						HANDLE ); // a handle of an Device previously opened
						
	// returns the resolution the analog card offers, 
	// or 0, 


// -------------------------------------------------------------------------
//					Functions for generic device setup:
// -------------------------------------------------------------------------

			//
			// This function sets a generic property of the device
BOOL DIIEXPORT DiiSetDeviceProperty (	HANDLE,		// a handle to a device previously opened
										LPCTSTR,	// a pointer to a string containg the property to set
										LPCTSTR		// a pointer to a string containing the new property value to set
									);

			//
			// This function gets a generic property of the device
BOOL DIIEXPORT DiiGetDeviceProperty (	HANDLE,		// a handle to a device previously opened
										LPCTSTR,	// a pointer to a string containg the property to read
										LPTSTR,		// a pointer to a string receiving the current value of the property
										DWORD
									);

// -------------------------------------------------------------------------
//			Functions to for digital input/output
// -------------------------------------------------------------------------


// DiiSetDigitalBit sets a digital bit on a 
// channel

BOOL DIIEXPORT DiiSetDigitalBit(
						HANDLE ,	// a handle of an Device previously opened
						DWORD ,		// the number of the port to write
						BOOL		// the value to write to the channel
						);
	// return value: TRUE if the value was written
	// successfully, FALSE otherwise.
	// see GetLastError error codes for more detailed information


// DiiSetDigitalByte writes an 8-bit value to a digital
// channel

BOOL DIIEXPORT DiiSetDigitalByte (
						HANDLE,		// a handle of an Device previously opened
						DWORD,		// the number of the port to write
						BYTE);		// the value to write to the channel

	// return value: TRUE if the value was written
	// successfully, FALSE otherwise.
	// see GetLastError error codes for more detailed information

// DiiGetDigitalBit reads an 1-bit value from a digital
// channel

BOOL DIIEXPORT DiiGetDigitalBit(
							HANDLE ,	// a handle of an Device previously opened
							DWORD ,		// the number of the port to write
							LPBOOL		// the value to write to the channel
							);
	// return value: TRUE if the value was written
	// successfully, FALSE otherwise.
	// see GetLastError error codes for more detailed information


// DiiGetDigitalByte reads an 8-bit value from a digital
// channel

BOOL DIIEXPORT DiiGetDigitalByte (
						HANDLE,		// a handle of an Device previously opened
						DWORD,		// the number of the port to read
						LPBYTE);	// points to the value read from the port

	// return value: TRUE if the value was read
	// successfully, FALSE otherwise.
	// see GetLastError error codes for more detailed information

//
// DiiGetOutputPortByte reads the state of an output port, either
// directly from the card, or from the port cache of the kernel drivers
// (if any)
//

BOOL DIIEXPORT DiiGetOutputPort (
						HANDLE,		// a handle of an Device previously opened
						DWORD,		// the number of the output port to read
						LPBYTE);	// points to the value read from the port

// -------------------------------------------------------------------------
//			Functions to configure digital channels.
// -------------------------------------------------------------------------

// For 8255 Cards, you may also set the chip configuration
// directly using DiiSetChipConfig.

BOOL DIIEXPORT DiiSet8255Config (
						HANDLE,		// a handle of an Device previously opened
						DWORD,		// the index of the chip to configure
						BYTE);		// the new configuration byte for the chip

	// return value: TRUE the chip was reconfigured
	// successfully, FALSE otherwise
	// see GetLastError error codes for more detailed information

// -------------------------------------------------------------------------
//			Functions to for analog input / output
// -------------------------------------------------------------------------

// DiiSetAnalogChannel writes a value to be output
// on the analog channel.

BOOL DIIEXPORT DiiSetAnalogChannel (
						HANDLE,		// a handle of an Device previously opened
						DWORD,		// the index of the channel to write to
						DWORD);		// the value to write to the channel

	// return value: TRUE if the value was written
	// successfully, FALSE otherwise.
	// see GetLastError error codes for more detailed information
						
// DiiReadAnalog reads a value from a specific 
// analog channel

BOOL DIIEXPORT DiiGetAnalogChannel (
						HANDLE,		// a handle of an Device previously opened
						DWORD,		// the number of the channel to read
						LPDWORD);	// points to the value read from the channel

	// return value: TRUE if the value was read
	// successfully, FALSE otherwise.
	// see GetLastError error codes for more detailed information


	// --------------------------------------------
	// Extended Analog interface added with V1.7.0:
	// --------------------------------------------


			//
			// These function setup a (programmable) gain for
			// an adapter, either for input or output channels.
BOOL DIIEXPORT DiiSetChannelGain (
						HANDLE,		// a handle of a device previously opened
						BOOL,		// indicates, whether an input (TRUE) or output (FALSE) channel is set
						DWORD,		// indicates the channel number
						double );	// indicates the channel gain to be set

BOOL DIIEXPORT DiiGetChannelGain (
						HANDLE,		// a handle of a device previously opened
						BOOL,		// indicates, whether an input (TRUE) or output (FALSE) channel is set
						DWORD,		// indicates the channel number
						double * );	// indicates the channel gain to be set

			//
			// Sets the uni-polar or bi-polar mode per channel:
BOOL DIIEXPORT DiiSetChannelBipolar (
						HANDLE,		// a handle of a device previously opened
						BOOL,		// indicates, whether an input (TRUE) or output (FALSE) channel is set
						DWORD,		// indicates the channel number
						BOOL );		// if TRUE, the channel is set to bipolar, FALSE otherwise

BOOL DIIEXPORT DiiGetChannelBipolar (
						HANDLE,		// a handle of a device previously opened
						BOOL,		// indicates, whether an input (TRUE) or output (FALSE) channel is set
						DWORD,		// indicates the channel number
						BOOL *);	// if TRUE, the channel is set to bipolar, FALSE otherwise
			//
			// Set the range that is being captured on the channel.
			// This range is user configured and is adjusted to the 
			// AD-range the device physically supports.
BOOL DIIEXPORT DiiSetChannelRange (
						HANDLE,		// a handle of a device previously opened
						BOOL,		// indicates, whether an input (TRUE) or output (FALSE) channel is set
						DWORD,		// indicates the channel number
						double,		// specifies the minimum value for the channel
						double);	// specifies the maximum value for the channel

BOOL DIIEXPORT DiiGetChannelRange (
						HANDLE,		// a handle of a device previously opened
						BOOL,		// indicates, whether an input (TRUE) or output (FALSE) channel is set
						DWORD,		// indicates the channel number
						double *,	// receives the minimum value for the channel
						double *);	// receives the maximum value for the channel

			// 
			// This function returns the range of AD values that are available
			// for this channel. It takes into account the physical limitations
			// of the AD device, as well as the currently selected gain for
			// the specified channel.
BOOL DIIEXPORT DiiGetDeviceChannelRange (
						HANDLE,		// a handle of a device previously opened
						BOOL,		// indicates, whether an input (TRUE) or output (FALSE) channel is set
						DWORD,		// indicates the channel number
						double *,	// receives the physical minimum value for the channel
						double *);	// receives the physical maximum value for the channel


			//
			// These functions set extended properties for a channel:
BOOL DIIEXPORT DiiSetExtendedChannelProperty (
						HANDLE,		// a handle of a device previously opened
						BOOL,		// indicates, whether an input (TRUE) or output (FALSE) channel is set
						DWORD,		// indicates the channel number
						LPCTSTR,	// contains the name of the property to set
						double);	// contains the new value of the property

BOOL DIIEXPORT DiiGetExtendedChannelProperty (
						HANDLE,		// a handle of a device previously opened
						BOOL,		// indicates, whether an input (TRUE) or output (FALSE) channel is set
						DWORD,		// indicates the channel number
						LPCTSTR,	// contains the name of the property to set
						double *);	// contains the new value of the property

			//
			// This function is called to enumerate properties that are supported
			// by this class
BOOL DIIEXPORT DiiEnumExtendedProperties (
								HANDLE,		//
								int,		// the index of the property name to retrieve
								LPCSTR,	// a pointer to a buffer receiving the property name
								UINT);		// the maximum size of the buffer above.


			//
			// These functions set and retrieve a single AD channel value from the card
BOOL DIIEXPORT DiiSetRealAnalogChannel (
						HANDLE,		// a handle of a device previously opened
						DWORD,		// indicates the channel number
						double);	// the new value of the channel

BOOL DIIEXPORT DiiGetRealAnalogChannel (
						HANDLE,		// a handle of a device previously opened
						DWORD,		// indicates the channel number
						double *);	// the current value of the channel

// -------------------------------------------------------------------------
//			Functions to access timers on cards:
// -------------------------------------------------------------------------

//
// The timer configuration to set:
//

BOOL DIIEXPORT DiiSetTimerConfig (
						HANDLE,		// a handle of an Device previously opened
						DWORD,		// the index of the timer to configure
						DWORD);		// the configuration of the timer

	// return value: TRUE if the timer was configured 
	// successfully, FALSE otherwise.
	// see GetLastError error codes for more detailed information

// DiiLoadTimer loads a value into a specific timer

BOOL DIIEXPORT DiiLoadTimer (
						HANDLE,		// a handle of an Device previously opened
						DWORD,		// the index of the timer to configure
						DWORD);		// the value to load into the timer

	// return value: TRUE if the timer loaded
	// successfully, FALSE otherwise.
	// see GetLastError error codes for more detailed information

// DiiGetTimer returns the current value of the timer

BOOL DIIEXPORT DiiGetTimer (
						HANDLE,		// a handle of an Device previously opened
						DWORD,		// the index of the timer to configure
						LPDWORD);	// points to a variable receiving the current value of the timer

// -------------------------------------------------------------------------
//			Functions to handle standard notification:
// -------------------------------------------------------------------------

// Notification types:

#define DII_TIMER_EXPIRED 0x000000001
	//		Can be called, when a timer has expired
#define DII_SIGNAL_CHANGED 0x00000002
	//		Can be called, whenever signals on the card have changed
#define DII_DATA_BLOCK_READ 0x00000004
	//		Can be called, when background data is available
#define DII_DATA_BLOCK_SENT 0x00000008
	//		Can be called, as soon as a data block has been sent
#define DII_VALUE_OUT_OF_RANGE 0x00000010
	//		(reserved for future use)
#define DII_COUNTER_ZERO 0x00000020
	//		Can be called when a counter has reached zero
#define DII_NOTIFICATION_USER 0x8000000
	//		Is used internally by the driver DLL.


	//
	// The following data structure is send along with a notification
	// to the user application:
	//

typedef struct _tagDII_NOTIFICATION_DATA
{
	HANDLE	hDevice;			// Identifies the device, which has issued the notification

	DWORD	dwNotificationMask;	// A bitmask identifying the notification
								// request this data is for.

	DWORD	dwNotificationParam;	
							// a 32-bit parameter passed along with the
							// notification. 
							// This might contain the port index that has changed
							// or the counter that has reached zero, etc.

	LPVOID	lpBuffer;		// The buffer containing event data, or NULL
	DWORD	dwBufferSize;	// The size of the buffer pointed to by lpBuffer

	DWORD	dwUserData;		// a 32-bit integer for user data (passed into 
							// DiiRequestNotification

	DWORD	dwAllocationScheme;	// Internal: Allocation scheme used for this
							// notification data block.

} DII_NOTIFICATION_DATA, * LPDII_NOTIFICATION_DATA;


typedef void (*PDII_NOTIFY_CALLBACK)(LPDII_NOTIFICATION_DATA);

	//
	// DiiSetNotificationMethodHwnd registers a windows message with
	// a device, that is posted, whenever the device needs to notify
	// the application about some event happening.
	//


BOOL DIIEXPORT DiiSetNotificationMethodHwnd (
							HANDLE ,// a handle of an Device previously opened
							HWND,	// a handle of the window, where to post the message to
							UINT,	// a message to send to the application.
									// Note that the LPARAM parameter of the message
									// will be a pointer to the DII_NOTIFICATION_DATA
									// structure
							WPARAM	// the wparam to send
							);	

	//
	// DiiSetNotificationMethodCallBack registers a callback routine that get's
	// called whenever a notification needs to be sent.
	// The function will be called in a different thread than the calling
	// thread, so the user application has to take care of synchronization
	// itself
	//

BOOL DIIEXPORT DiiSetNotificationMethodCallBack (
								HANDLE ,	// a handle of an Device previously opened
								PDII_NOTIFY_CALLBACK
											// a pointer to a callback routine
								);

	//
	// DiiRequestNotification is called to request a notification.
	// Previous to calling this routine, a notification method should have
	// been installed with any of the two above calls.
	// The calling application will receive notifications for any of these
	// two events.
	//
BOOL DIIEXPORT DiiRequestNotification (	HANDLE, // a handle of an Device previously opened
								DWORD ,	// a mask of notification ID, which the
										// application requests
								DWORD 	// A user value that get's passed back
										// upon every notification
							);

	//
	// DiiCancelNotification is called to cancel a previously requested
	// notification (with DiiRequestNotification) again.
	//
BOOL DIIEXPORT DiiCancelNotification (
							HANDLE ,	// a handle of an Device previously opened
							DWORD		// a mask of notification ID, which the
										// application requests
							);

	//
	// DiiReleaseNotificationData (HANDLE,LPDII_NOTIFICATION_DATA) 
	// needs to be called to when notifications are sent to a window message,
	// to release the notification structure given. Calling it on the notification
	// data structure passed into the DII_NOTIFY_CALLBACK function has no effect.
	//
BOOL DIIEXPORT DiiReleaseNotificationData (	
							HANDLE,
							LPDII_NOTIFICATION_DATA
						);

// -------------------------------------------------------------------------
//							Pulse Counting Interface:
//		This interface implements DII pulse counting automatically for
//		all cards having Digital I/O
// -------------------------------------------------------------------------


// This function enables pulse counting for the specified card.
// All channels are by default up-counter.
BOOL DIIEXPORT DiiEnablePulseCounting ( 
								HANDLE hDevice, // 
								DWORD dwStartPort,
								DWORD dwNumberOfPorts,
								DWORD dwReserved
								);

// This function disables pulse counting for the specified card:

BOOL DIIEXPORT DiiDisablePulseCounting (HANDLE hDevice);

// 
// This function configures a counter to be a downcounter
//

BOOL DIIEXPORT DiiSetDownPulseCounter (	HANDLE hDevice,
										DWORD dwChannel,
										DWORD dwInitialValue,
										BOOL bAutoReset
										);
//
// This function retrieves the current value of an up or downcounter
//
BOOL DIIEXPORT DiiGetPulseCounterValue (	HANDLE hDevice,
											DWORD dwChannel,
											LPDWORD lpdwValue,
											LPDWORD lpdwMinTimeBetweenPulses,
											LPDWORD lpdwMaxTimeBetweenPulses,
											BOOL bResetCounter
										);

// -------------------------------------------------------------------------
//							Block Asynch Interface
//	
//  This interface implements block based I/O:
// -------------------------------------------------------------------------

	//
	// These define the trigger sources for a performing a data acquisition

#define DII_TRIGGER_SOFTWARE		0
#define DII_TRIGGER_TIMER			1
#define DII_TRIGGER_DIGITAL_CHANNEL 2

// 
// This function is used to prepare the block input for a specific channel.
// It needs to be called once for every channel that shall be acquired
//
BOOL DIIEXPORT DiiPrepareBlockInput	(	HANDLE hDevice,			// the device handle
										DWORD dwChannel,		// the channel to acquire
										DWORD dwTriggerSource,	// the trigger source (see above)
										DWORD dwTriggerValue,	// i.e. frequency or digital input channel used for triggering
										DWORD dwSampleBufferSize,	// requested buffer size for sampling data.
										LPCTSTR lpszFileName // target file
									);

//
// This function is called to actually start the background input of data blocks
// from the specified channels. Acquisition will start in a background thread
BOOL DIIEXPORT DiiStartBlockInput(		HANDLE hDevice );

//
// This function is called to stop the background block input of data
BOOL DIIEXPORT DiiEndBlockInput (		HANDLE hDevice );

//
// This function returns information about the block input for a 
// specific channel
BOOL DIIEXPORT DiiGetBlockInputStatus	(	HANDLE hDevice,		// the device
											DWORD dwChannel,	// the channel for which information is to be retrieved
											LPDWORD lpdwTotalSamples,	// the total number of samples acquired so far
											LPDWORD lpdwSamplesPerSecond,	// the acquired samples per second
											LPDWORD lpdwBufferUnderruns	// the number of buffer underruns encountered
										);

//
// This function opens a block input file that was captured to disk
// 
BOOL DIIEXPORT DiiOpenBlockInputFile	(	
											HANDLE hDevice,		// the device under consideration
											DWORD dwChannel,	// the channel for which the file is opened
											LPCTSTR lpszFileName// the filename of the dump file
										);
//
// This function reads a block of data from the input file
BOOL DIIEXPORT DiiReadBlockInputFile	(	
											HANDLE hDevice,		// the device
											DWORD dwChannel,	// the channel to read
											LPVOID lpvBuffer,	// the destination buffer for data samples
											DWORD dwBufferSize,	// the buffersize available in 'lpvBuffer'
											LPDWORD dwBufferSizeFilled	// the buffersize actually filled
										);	
//
// This function is called to close a block input file again
BOOL DIIEXPORT DiiCloseBlockInputFile (		HANDLE hDevice,		// the device
											DWORD dwChannel		// the channel to be closed
										);


//
// Block output functions:

//
// This function is called to prepare block output for a specific channel
BOOL DIIEXPORT DiiPrepareBlockOutput (	HANDLE hDevice,			// the device
										DWORD dwChannel,		// the channel for block output
										DWORD dwTriggerSource,	// the trigger source (see above)
										DWORD dwTriggerValue	// i.e. frequency or digital input channel used for triggering
									);

//
// This function actually starts block output on the channels
// previously initialized via "DiiPrepareBlockOutput"
BOOL DIIEXPORT DiiStartBlockOutput	(	HANDLE hDevice);	


//
// This functio is called to perform a block output on the given channel.
// Note that it will perform this output asynchronous, that is in the background.
// the buffer must be accessible by the background routine.
BOOL DIIEXPORT DiiOutputBlock	(	HANDLE hDevice,			// the device
									DWORD dwChannel,		// the channel
									LPVOID lpvBuffer,		// the buffer to be output
									DWORD dwBufferSize,		// the buffer size
									BOOL bContinueous		// TRUE if the output shall be sent contineously, FALSE otherwise
								);

//
// This function is called to output data from the specified file to
// the channel. The data must be previously acquired by the block input routines
BOOL DIIEXPORT DiiOutputBlockFromFile
								(	HANDLE hDevice,			// the device
									DWORD dwChannel,		// the channel
									LPCTSTR lpszFileName,	// the name of the file to be 'played'
									BOOL bContinueous		// TRUE if the output shall be contineously output
								);

//
// This function is called to end the block output machanism
BOOL DIIEXPORT DiiEndBlockOutput ( HANDLE hDevice );

//
// This function returns information about the block output
BOOL DIIEXPORT DiiGetBlockOutputStatus	(	
										HANDLE hDevice,
										DWORD dwChannel,	// the channel for which information is to be retrieved
										LPDWORD lpdwTotalSamples,	// the total number of samples acquired so far
										LPDWORD lpdwSamplesPerSecond,	// the acquired samples per second
										LPDWORD lpdwBufferUnderruns	// the number of buffer underruns encountered
									);



// -------------------------------------------------------------------------
//							Fast I/O Interface:
//	This interface is implemented to provide fast I/O access to cards with
//  I/O ports. 
// -------------------------------------------------------------------------

		//
		// This function enables fast output to a port on the device
BOOL DIIEXPORT DiiFastIoOutputPort (	HANDLE hDevice,
										DWORD dwPort,
										PVOID lpvBuffer,
										DWORD dwBufferLength 
								);

		//
		// This function enables fast input from a port on the device

BOOL DIIEXPORT DiiFastIoInputPort (		HANDLE hDevice,
										DWORD dwPort,
										PVOID lpvBuffer,
										DWORD dwBufferLength 
								);

#if defined (__cplusplus)
}
#endif


#endif
